/*************************************************************
Module name: CopyData.C
Notices: Copyright (c) 1995 Jeffrey Richter
*************************************************************/


#include "..\AdvWin32.H"     /* See Appendix B for details. */
#include <windows.h>
#include <windowsx.h>

#pragma warning(disable: 4001)       /* Single-line comment */

#include <tchar.h>
#include "Resource.H"


//////////////////////////////////////////////////////////////


// Microsoft does not include message cracker macros for 
// the WM_COPYDATA message in WINDOWSX.H. 
// I have written them here....

/* BOOL Cls_OnCopyData(HWND hwnd, HWND hwndFrom, 
      PCOPYDATASTRUCT cds) */

#define HANDLE_WM_COPYDATA(hwnd, wParam, lParam, fn) \
    ((fn)((hwnd), (HWND)(wParam), \
     (PCOPYDATASTRUCT)lParam), 0L)

#define FORWARD_WM_COPYDATA(hwnd, hwndFrom, cds, fn) \
    (BOOL)(UINT)(DWORD)(fn)((hwnd), WM_COPYDATA, \
      (WPARAM)(hwndFrom), (LPARAM)(cds))


/////////////////////////////////////////////////////////////


BOOL Dlg_OnCopyData(HWND hwnd, HWND hwndFrom, 
   PCOPYDATASTRUCT cds) {
   
   Edit_SetText(
      GetDlgItem(hwnd, cds->dwData ? IDC_DATA2 : IDC_DATA1), 
      cds->lpData);
   return(TRUE);
}


//////////////////////////////////////////////////////////////


BOOL Dlg_OnInitDialog (HWND hwnd, HWND hwndFocus, 
   LPARAM lParam) {

    // Associate an icon with the dialog box.
    SetClassLong(hwnd, GCL_HICON, (LONG)
      LoadIcon((HINSTANCE) GetWindowLong(hwnd, GWL_HINSTANCE),
      __TEXT("CopyData")));

   // Initialize the edit control with some test data.
   Edit_SetText(GetDlgItem(hwnd, IDC_DATA1), 
      __TEXT("Some test data"));
   Edit_SetText(GetDlgItem(hwnd, IDC_DATA2), 
      __TEXT("Some more test data"));

   return(TRUE);
}


//////////////////////////////////////////////////////////////


void Dlg_OnCommand (HWND hwnd, int id, HWND hwndCtl, 
   UINT codeNotify) {

   HWND hwndEdit, hwndSibling;
   COPYDATASTRUCT cds;
   TCHAR szCaption[100], szCaptionSibling[100];

   switch (id) {
      case IDC_COPYDATA1:
      case IDC_COPYDATA2:
         if (codeNotify != BN_CLICKED)
            break;

         hwndEdit = GetDlgItem(hwnd,
            (id == IDC_COPYDATA1) ? IDC_DATA1 : IDC_DATA2);

         // Prepare the contents of the COPYDATASTRUCT.
         // 0 = ID_DATA1, 1 = ID_DATA2
         cds.dwData = (DWORD) ((id == IDC_COPYDATA1) ? 0 : 1);

         // Get the length of the data block 
         // that we are sending.
         cds.cbData = (Edit_GetTextLength(hwndEdit) + 1) * 
            sizeof(TCHAR);

         // Allocate a block of memory to hold the string.
         cds.lpData = HeapAlloc(GetProcessHeap(),
            HEAP_ZERO_MEMORY, cds.cbData);

         // Put the edit control's string in the data block.
         Edit_GetText(hwndEdit, cds.lpData, cds.cbData);

         // Find the first overlapped window in the list.
         hwndSibling = GetFirstSibling(hwnd);

         // Get the caption of our window.
         GetWindowText(hwnd, szCaption, 
            ARRAY_SIZE(szCaption));

         while (IsWindow(hwndSibling)) {
            // Get the caption of the potential 
            // window to send the data to.
            GetWindowText(hwndSibling, szCaptionSibling,
               ARRAY_SIZE(szCaptionSibling));

            if (_tcscmp(szCaption, szCaptionSibling) == 0) {
               // If the window's caption is the same as ours, 
               // send the data. This may mean that we are 
               // sending the message to ourselves. This is OK;
               // it demonstrates that WM_COPYDATA can be 
               // used to send data to ourselves.
               FORWARD_WM_COPYDATA(hwndSibling, hwnd, 
                  &cds, SendMessage);
            }

            // Get the handle of the next overlapped window.
            hwndSibling = GetNextSibling(hwndSibling);
         }

         // Free the data buffer.
         HeapFree(GetProcessHeap(), 0, cds.lpData);
         break;

      case IDCANCEL:
         EndDialog(hwnd, id);
         break;
   }
}


//////////////////////////////////////////////////////////////


BOOL CALLBACK Dlg_Proc (HWND hDlg, UINT uMsg, 
   WPARAM wParam, LPARAM lParam) {
   
   BOOL fProcessed = TRUE;

   switch (uMsg) {
      HANDLE_MSG(hDlg, WM_INITDIALOG, Dlg_OnInitDialog);
      HANDLE_MSG(hDlg, WM_COMMAND, Dlg_OnCommand);
      HANDLE_MSG(hDlg, WM_COPYDATA, Dlg_OnCopyData);

      default:
         fProcessed = FALSE;
         break;
   }
   return(fProcessed);
}


//////////////////////////////////////////////////////////////


int WINAPI WinMain (HINSTANCE hinstExe,
   HINSTANCE hinstPrev, LPSTR lpszCmdLine, int nCmdShow) {

   DialogBox(hinstExe, MAKEINTRESOURCE(IDD_COPYDATA), 
      NULL, Dlg_Proc);

   return(0);
}


/////////////////////// End Of File ////////////////////////
