/************************************************************
Module name: DiskInfo.C
Notices: Copyright (c) 1995 Jeffrey Richter
************************************************************/


#include "..\AdvWin32.H"    /* See Appendix B for details. */
#include <windows.h>
#include <windowsx.h>

#pragma warning(disable: 4001)      /* Single-line comment */

#include <tchar.h>
#include <stdio.h>             // For sprintf
#include <string.h>            // For strchr
#include "Resource.H"


/////////////////////////////////////////////////////////////


void Dlg_FillDriveInfo (HWND hwnd, LPTSTR lpszRootPathName) {
   // Variables for processing the drive type information.
   int nDriveType;
   LPCTSTR p;

   // Variables for processing the volume information
   TCHAR szBuf[200];
   TCHAR lpVolumeNameBuffer[200];
   DWORD dwVolumeSerialNumber, dwMaximumComponentLength;
   DWORD dwFileSystemFlags;
   TCHAR lpFileSystemNameBuffer[50];

   // Variables for processing the disk space information
   DWORD dwSectorsPerCluster, dwBytesPerSector;
   DWORD dwFreeClusters, dwClusters;


   // Get the drive type information.
   nDriveType = GetDriveType(lpszRootPathName);
   switch (nDriveType) {
      case 0:
         p = __TEXT("Cannot be determined.");
         break;

      case 1:
         p = __TEXT("Path does not exist.");
         break;

      case DRIVE_REMOVABLE:
         p = __TEXT("Removable");
         break;

      case DRIVE_FIXED:
         p = __TEXT("Fixed");
         break;

      case DRIVE_REMOTE:
         p = __TEXT("Remote");
         break;

      case DRIVE_CDROM:
         p = __TEXT("CD-ROM");
         break;

      case DRIVE_RAMDISK:
         p = __TEXT("RAM disk");
         break;

      default:
         p = __TEXT("Unknown");
         break;
   }

   SetWindowText(GetDlgItem(hwnd, IDC_DRIVETYPE), p);



   // Get the volume information.
   if (GetVolumeInformation(lpszRootPathName,
      lpVolumeNameBuffer,
      ARRAY_SIZE(lpVolumeNameBuffer), &dwVolumeSerialNumber,
      &dwMaximumComponentLength, &dwFileSystemFlags,
      lpFileSystemNameBuffer,
      ARRAY_SIZE(lpFileSystemNameBuffer))) {

      _stprintf(szBuf, __TEXT("%s\n%u\n%u\n"),
         lpVolumeNameBuffer, dwVolumeSerialNumber,
         dwMaximumComponentLength);

      if (dwFileSystemFlags & FS_CASE_IS_PRESERVED)
         _tcscat(szBuf, __TEXT("FS_CASE_IS_PRESERVED"));
      _tcscat(szBuf, __TEXT("\n"));

      if (dwFileSystemFlags & FS_CASE_SENSITIVE)
         _tcscat(szBuf, __TEXT("FS_CASE_SENSITIVE"));
      _tcscat(szBuf, __TEXT("\n"));

      if (dwFileSystemFlags & FS_UNICODE_STORED_ON_DISK)
         _tcscat(szBuf, __TEXT("FS_UNICODE_STORED_ON_DISK"));
      _tcscat(szBuf, __TEXT("\n"));

      if (dwFileSystemFlags & FS_PERSISTENT_ACLS)
         _tcscat(szBuf, __TEXT("FS_PERSISTENT_ACLS"));
      _tcscat(szBuf, __TEXT("\n"));

      _tcscat(szBuf, lpFileSystemNameBuffer);
   } else {
      _tcscpy(szBuf, __TEXT("NO VOLUME INFO"));
   }
   SetWindowText(GetDlgItem(hwnd, IDC_VOLINFO), szBuf);


   // Get the disk space information.
   if (GetDiskFreeSpace(lpszRootPathName,
      &dwSectorsPerCluster, &dwBytesPerSector,
      &dwFreeClusters, &dwClusters)) {

      _stprintf(szBuf, __TEXT("%u\n%u\n%u\n%u"),
         dwSectorsPerCluster, dwBytesPerSector,
         dwFreeClusters, dwClusters);
   } else {
      _tcscpy(szBuf, __TEXT("NO\nDISK\nSPACE\nINFO"));
   }
   SetWindowText(GetDlgItem(hwnd, IDC_DISKINFO), szBuf);
}


/////////////////////////////////////////////////////////////


BOOL Dlg_OnInitDialog (HWND hwnd, HWND hwndFocus,
   LPARAM lParam) {

   DWORD  dwNumBytesForDriveStrings;
   HANDLE hHeap;
   LPTSTR lp;
   TCHAR  szLogDrive[100];
   HWND   hwndCtl = GetDlgItem(hwnd, IDC_LOGDRIVES);
   int    nNumDrives = 0, nDriveNum;

   // Associate an icon with the dialog box.
   SetClassLong(hwnd, GCL_HICON, (LONG)
      LoadIcon((HINSTANCE) GetWindowLong(hwnd, GWL_HINSTANCE),
      __TEXT("DiskInfo")));

   // Get the number of bytes needed to hold all
   // the logical drive strings.
   dwNumBytesForDriveStrings =
      GetLogicalDriveStrings(0, NULL) * sizeof(TCHAR);

   if (dwNumBytesForDriveStrings != 0) {
      // The GetLogicalDriveStrings function is 
      // supported on this platform.

      // Allocate memory from the heap for the drive
      // string names.
      hHeap = GetProcessHeap();
      lp = (LPTSTR) HeapAlloc(hHeap, HEAP_ZERO_MEMORY,
         dwNumBytesForDriveStrings);

      // Get the drive string names in our buffer.
      GetLogicalDriveStrings(HeapSize(hHeap, 0, lp), lp);

      // Parse the memory block, and fill the combo box.
      while (*lp != 0) {
         ComboBox_AddString(hwndCtl, lp);
         nNumDrives++;
         lp = _tcschr(lp, 0) + 1;   // Point to next string.
      }

      HeapFree(hHeap, 0, lp);
   } else {
      // The GetLogicalDriveStrings function is NOT
      // supported on this platform--Windows 95.
      DWORD dwDriveMask = GetLogicalDrives();
      for (nDriveNum = 0; dwDriveMask != 0; ) {
         wsprintf(szLogDrive, __TEXT("%c:\\"), 
            (TCHAR) (__TEXT('A') + nDriveNum));
            ComboBox_AddString(hwndCtl, szLogDrive);

         nDriveNum++;         // Increment the drive number.
         dwDriveMask >>= 1;   // Check next bit.
      }
   }

   // Initialize the volume information for the first fixed
   // drive so that we don't try to read volume
   // information from a drive that doesn't contain a
   // diskette.
   for (nDriveNum = 0; nDriveNum < nNumDrives; nDriveNum++) {
      ComboBox_GetLBText(hwndCtl, nDriveNum, szLogDrive);
      if (GetDriveType(szLogDrive) == DRIVE_FIXED)
         break;
   }

   if (nDriveNum == nNumDrives) {
      // There are no fixed drives--just use the
      // first drive.
      ComboBox_GetLBText(hwndCtl, nDriveNum = 0, szLogDrive);
   }

   // Select the first fixed drive, or select the first
   // drive if no fixed drives exist.
   ComboBox_SetCurSel(hwndCtl, nDriveNum);

   Dlg_FillDriveInfo(hwnd, szLogDrive);

   return(TRUE);
}


/////////////////////////////////////////////////////////////


void Dlg_OnCommand (HWND hwnd, int id, HWND hwndCtl,
   UINT codeNotify) {

   TCHAR szLogDrive[100];

   switch (id) {
      case IDC_LOGDRIVES:
         if (codeNotify != CBN_SELCHANGE)
            break;

         ComboBox_GetText(hwndCtl, szLogDrive,
            ARRAY_SIZE(szLogDrive));
         Dlg_FillDriveInfo(hwnd, szLogDrive);
         break;

      case IDCANCEL:
         EndDialog(hwnd, id);
         break;
   }
}


/////////////////////////////////////////////////////////////


BOOL CALLBACK Dlg_Proc (HWND hDlg, UINT uMsg,
   WPARAM wParam, LPARAM lParam) {

   BOOL fProcessed = TRUE;

   switch (uMsg) {
      HANDLE_MSG(hDlg, WM_INITDIALOG, Dlg_OnInitDialog);
      HANDLE_MSG(hDlg, WM_COMMAND, Dlg_OnCommand);

      default:
         fProcessed = FALSE;
         break;
   }
   return(fProcessed);
}


/////////////////////////////////////////////////////////////


int WINAPI WinMain (HINSTANCE hinstExe,
   HINSTANCE hinstPrev, LPSTR lpszCmdLine, int nCmdShow) {

   DialogBox(hinstExe, MAKEINTRESOURCE(IDD_DISKINFO),
      NULL, Dlg_Proc);

   return(0);
}


//////////////////////// End Of File ////////////////////////
