/*************************************************************
Module name: DocStats.C
Notices: Copyright (c) 1995 Jeffrey Richter
*************************************************************/


#include "..\AdvWin32.H"     /* See Appendix B for details. */
#include <windows.h>
#include <windowsx.h>

#pragma warning(disable: 4001)       /* Single-line comment */

#include <tchar.h>
#include <stdio.h>             // For sprintf
#include <string.h>
#include <process.h>           // For _beginthreadex
#include "Resource.H"


//////////////////////////////////////////////////////////////


typedef enum {
   STAT_FIRST = 0,
   STAT_LETTERS = STAT_FIRST,
   STAT_WORDS,
   STAT_LINES,
   STAT_LAST = STAT_LINES
} STATTYPE;

HANDLE g_hEventsDataReady[STAT_LAST - STAT_FIRST + 1];
HANDLE g_hEventsProcIdle[STAT_LAST - STAT_FIRST + 1];

BYTE g_bFileBuf[1024];
DWORD g_dwNumBytesInBuf;

DWORD WINAPI LetterStats (LPVOID lpvParam);
DWORD WINAPI WordStats (LPVOID lpvParam);
DWORD WINAPI LineStats (LPVOID lpvParam);


//////////////////////////////////////////////////////////////


int WINAPI WinMain(HINSTANCE hinstExe,
   HINSTANCE hinstPrev, LPSTR lpszCmdLine, int nCmdShow) {

   HANDLE hThreads[STAT_LAST - STAT_FIRST + 1];
   HANDLE hFile;
   DWORD dwNumLetters = 0, dwNumWords = 0, dwNumLines = 0;
   DWORD dwThreadID;
   TCHAR szBuf[150];
   LPTSTR lpszCmdLineT;


   // Get the name of the file.
   // We must use GetCommandLine here instead of WinMain's 
   // lpszCmdLine parameter because lpszCmdLine is always an 
   // ANSI string -- never a Unicode string. GetCommandLine 
   // returns ANSI or Unicode depending on how we 
   // have compiled.
   lpszCmdLineT = _tcschr(GetCommandLine(), __TEXT(' '));

   if (lpszCmdLineT != NULL) {
      // We found a space after the executable's filename.  
      // Now let's skip over any white space to get to the 
      // first argument.
      while (*lpszCmdLineT == __TEXT(' '))
         lpszCmdLineT++;
   }

   if ((lpszCmdLineT == NULL) || (*lpszCmdLineT == 0)) {
      // If a space was not found or there are no arguments 
      // after the executable's filename, display an 
      // error message.

      MessageBox(NULL, 
         __TEXT("You must enter a filename on ")
         __TEXT("the command line."),
         __TEXT("DocStats"), MB_OK);
      return(0);
   }


   // Open the file for reading.
   hFile = CreateFile(lpszCmdLineT, GENERIC_READ, 0,
      NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
   if (hFile == INVALID_HANDLE_VALUE) {
      // File open failed.
      MessageBox(NULL, __TEXT("File could not be opened."),
         __TEXT("DocStats"), MB_OK);
      return(0);
   }

   // Signaled when not processing buffer.
   g_hEventsDataReady[STAT_LETTERS] =
      CreateEvent(NULL, FALSE, FALSE, NULL);
   g_hEventsProcIdle[STAT_LETTERS] =
      CreateEvent(NULL, FALSE, TRUE, NULL);

   g_hEventsDataReady[STAT_WORDS] =
      CreateEvent(NULL, FALSE, FALSE, NULL);
   g_hEventsProcIdle[STAT_WORDS] =
      CreateEvent(NULL, FALSE, TRUE, NULL);

   g_hEventsDataReady[STAT_LINES] =
      CreateEvent(NULL, FALSE, FALSE, NULL);
   g_hEventsProcIdle[STAT_LINES] =
      CreateEvent(NULL, FALSE, TRUE, NULL);


   // Create all the threads.  Threads MUST be
   // created AFTER the event objects.
   hThreads[STAT_LETTERS] = BEGINTHREADEX(NULL, 0, 
      LetterStats, NULL, 0, &dwThreadID);
   hThreads[STAT_WORDS]   = BEGINTHREADEX(NULL, 0, 
      WordStats, NULL, 0, &dwThreadID);
   hThreads[STAT_LINES]   = BEGINTHREADEX(NULL, 0, 
      LineStats, NULL, 0, &dwThreadID);


   do {
      // Wait for the worker threads to be idle.
      WaitForMultipleObjects(STAT_LAST - STAT_FIRST + 1,
         g_hEventsProcIdle, TRUE, INFINITE);

      // Read part of the file into the global memory buffer.
      ReadFile(hFile, g_bFileBuf, ARRAY_SIZE(g_bFileBuf),
         &g_dwNumBytesInBuf, NULL);

      // Signal the works that the data is ready.
      SetEvent(g_hEventsDataReady[STAT_LETTERS]);
      SetEvent(g_hEventsDataReady[STAT_WORDS]);
      SetEvent(g_hEventsDataReady[STAT_LINES]);

   } while (g_dwNumBytesInBuf != 0);


   // All the statistics for the file have been accumulated;
   // time to clean up.
   CloseHandle(hFile);

   // Wait for all of the threads to return.
   WaitForMultipleObjects(STAT_LAST - STAT_FIRST + 1, hThreads,
      TRUE, INFINITE);

   GetExitCodeThread(hThreads[STAT_LETTERS], &dwNumLetters);
   CloseHandle(hThreads[STAT_LETTERS]);
   CloseHandle(g_hEventsDataReady[STAT_LETTERS]);
   CloseHandle(g_hEventsProcIdle[STAT_LETTERS]);

   GetExitCodeThread(hThreads[STAT_WORDS], &dwNumWords);
   CloseHandle(hThreads[STAT_WORDS]);
   CloseHandle(g_hEventsDataReady[STAT_WORDS]);
   CloseHandle(g_hEventsProcIdle[STAT_WORDS]);

   GetExitCodeThread(hThreads[STAT_LINES], &dwNumLines);
   CloseHandle(hThreads[STAT_LINES]);
   CloseHandle(g_hEventsDataReady[STAT_LINES]);
   CloseHandle(g_hEventsProcIdle[STAT_LINES]);

   _stprintf(szBuf,
      __TEXT("Num letters = %d, Num words = %d, ")
      __TEXT("Num lines = %d"),
      dwNumLetters, dwNumWords, dwNumLines);

   MessageBox(NULL, szBuf, __TEXT("DocStats"), MB_OK);
   return(0);
}


//////////////////////////////////////////////////////////////


DWORD WINAPI LetterStats(LPVOID lpvParam) {
   DWORD dwNumLetters = 0, dwByteIndex;
   BYTE bByte;

   do {
      // Wait for the data to be ready.
      WaitForSingleObject(g_hEventsDataReady[STAT_LETTERS], 
         INFINITE);

      dwByteIndex = 0;
      for (; dwByteIndex < g_dwNumBytesInBuf; dwByteIndex++) {

         bByte = g_bFileBuf[dwByteIndex];

         // This program works only on ANSI files. Regardless 
         // of whether we compile DocStats for ANSI or for
         // Unicode, we must always call the ANSI version of 
         // IsCharAlpha.
         if (IsCharAlphaA(bByte))
            dwNumLetters++;
      }

      // Data processed; signal that we are done.
      SetEvent(g_hEventsProcIdle[STAT_LETTERS]);
   } while (g_dwNumBytesInBuf > 0);

   return(dwNumLetters);
}


//////////////////////////////////////////////////////////////


DWORD WINAPI WordStats(LPVOID lpvParam) {
   DWORD dwNumWords = 0, dwByteIndex;
   BYTE bByte;
   BOOL fInWord = FALSE, fIsWordSep;

   do {
      // Wait for the data to be ready.
      WaitForSingleObject(g_hEventsDataReady[STAT_WORDS], 
         INFINITE);

      dwByteIndex = 0;
      for (; dwByteIndex < g_dwNumBytesInBuf; dwByteIndex++) {

         bByte = g_bFileBuf[dwByteIndex];

         // This program works only on ANSI files. Regardless
         // of whether we compile DocStats for ANSI or for
         // Unicode, we must always call the ANSI version 
         // of strchr.
         fIsWordSep = (strchr(" \t\n\r", bByte) != NULL);

         if (!fInWord && !fIsWordSep) {
            dwNumWords++;
            fInWord = TRUE;
         } else {
            if (fInWord && fIsWordSep) {
               fInWord = FALSE;
            }
         }
      }

      // Data processed; signal that we are done.
      SetEvent(g_hEventsProcIdle[STAT_WORDS]);
   } while (g_dwNumBytesInBuf > 0);

   return(dwNumWords);
}


//////////////////////////////////////////////////////////////


DWORD WINAPI LineStats (LPVOID lpvParam) {
   DWORD dwNumLines = 0, dwByteIndex;
   BYTE bByte;

   do {
      // Wait for the data to be ready.
      WaitForSingleObject(g_hEventsDataReady[STAT_LINES], 
         INFINITE);

      dwByteIndex = 0;
      for (; dwByteIndex < g_dwNumBytesInBuf; dwByteIndex++) {

         bByte = g_bFileBuf[dwByteIndex];

         // This program works only on ANSI files. Regardless 
         // of whether we compile DocStats for ANSI or for
         // Unicode, we must always compare the byte to an 
         // ANSI version of '\n'.
         if ('\n' == bByte)
            dwNumLines++;
      }

      // Data processed; signal that we are done.
      SetEvent(g_hEventsProcIdle[STAT_LINES]);

   } while (g_dwNumBytesInBuf > 0);

   return(dwNumLines);
}


//////////////////////// End Of File /////////////////////////
