/*************************************************************
Module name: LISLab.C
Notices: Copyright (c) 1995 Jeffrey Richter
*************************************************************/


#include "..\AdvWin32.H"      /* See Appendix B for details */
#include <windows.h>
#include <windowsx.h>

#pragma warning(disable: 4001)       /* Single-line comment */

#include <tchar.h>
#include <string.h>
#include <stdio.h>             // For sprintf
#include "Resource.H"


//////////////////////////////////////////////////////////////


#define TIMER_DELAY (1 * 1000) // 1 second * 1000 milliseconds

UINT  g_uTimerId = 1;
int   g_nEventId = 0;
DWORD g_dwEventTime = 0;
HWND  g_hwndSubject = NULL;
HWND  g_hwndPM = NULL;


//////////////////////////////////////////////////////////////


void CalcWndText (HWND hwnd, LPTSTR szBuf, int nLen) {
   TCHAR szClass[50], szCaption[50], szBufT[150];

   if (hwnd == (HWND) NULL) {
      _tcscpy(szBuf, __TEXT("(no window)"));
      return;
   }


   if (!IsWindow(hwnd)) {
      _tcscpy(szBuf, __TEXT("(invalid window)"));
      return;
   }


   GetClassName(hwnd, szClass, ARRAY_SIZE(szClass));
   GetWindowText(hwnd, szCaption, ARRAY_SIZE(szCaption));

   _stprintf(szBufT, __TEXT("[%s] %s"), (LPTSTR) szClass,
      (*szCaption == 0) ? (LPTSTR) __TEXT("(no caption)")
      : (LPTSTR) szCaption);
   _tcsncpy(szBuf, szBufT, nLen - 1);
   szBuf[nLen - 1] = 0; // Force zero-terminated string
}


//////////////////////////////////////////////////////////////


// To minimize stack use, one instance of WALKWINDOWTREEDATA
// is created as a local variable in WalkWindowTree() and a
// pointer to it is passed to WalkWindowTreeRecurse.

// Data used by WalkWindowTreeRecurse
typedef struct {
   HWND  hwndLB;        // Handle to the output list box
   HWND  hwndParent;    // Handle to the parent
   int   nLevel;        // Nesting depth
   int   nIndex;        // List box item index
   TCHAR szBuf[100];    // Output buffer
   int   iBuf;          // Index into szBuf
} WALKWINDOWTREEDATA, *LPWALKWINDOWTREEDATA;


void WalkWindowTreeRecurse (LPWALKWINDOWTREEDATA pWWT) {

   const int nIndexAmount = 2;
   HWND hwndChild;

   pWWT->nLevel++;

   if (!IsWindow(pWWT->hwndParent))
      return;

   for (pWWT->iBuf = 0;
      pWWT->iBuf < pWWT->nLevel * nIndexAmount; pWWT->iBuf++)
      pWWT->szBuf[pWWT->iBuf] = __TEXT(' ');

   CalcWndText(pWWT->hwndParent, &pWWT->szBuf[pWWT->iBuf],
      ARRAY_SIZE(pWWT->szBuf) - pWWT->iBuf);
   pWWT->nIndex = ListBox_AddString(pWWT->hwndLB, pWWT->szBuf);
   ListBox_SetItemData(pWWT->hwndLB, pWWT->nIndex,
      pWWT->hwndParent);

   hwndChild = GetFirstChild(pWWT->hwndParent);
   while (hwndChild != NULL) {
      pWWT->hwndParent = hwndChild;
      WalkWindowTreeRecurse(pWWT);
      hwndChild = GetNextSibling(hwndChild);
   }

   pWWT->nLevel--;
}


//////////////////////////////////////////////////////////////


void WalkWindowTree (HWND hwndLB, HWND hwndParent) {

   WALKWINDOWTREEDATA WWT;

   WWT.hwndLB = hwndLB;
   WWT.hwndParent = hwndParent;
   WWT.nLevel = -1;

   WalkWindowTreeRecurse(&WWT);
}


//////////////////////////////////////////////////////////////


BOOL Dlg_OnInitDialog (HWND hwnd, HWND hwndFocus,
   LPARAM lParam) {

   HWND hwndT;

    // Associate an icon with the dialog box.
    SetClassLong(hwnd, GCL_HICON, (LONG)
      LoadIcon((HINSTANCE) GetWindowLong(hwnd, GWL_HINSTANCE),
      __TEXT("LISLab")));

   g_uTimerId = SetTimer(hwnd, g_uTimerId, TIMER_DELAY, NULL);

   hwndT = GetDlgItem(hwnd, IDC_WNDFUNC);
   ComboBox_AddString(hwndT, __TEXT("SetFocus"));
   ComboBox_AddString(hwndT, __TEXT("SetActiveWindow"));
   ComboBox_AddString(hwndT, __TEXT("SetForegroundWnd"));
   ComboBox_AddString(hwndT, __TEXT("BringWindowToTop"));
   ComboBox_AddString(hwndT, __TEXT("SetWindowPos-TOP"));
   ComboBox_AddString(hwndT, __TEXT("SetWindowPos-BTM"));
   ComboBox_SetCurSel(hwndT, 0);


   // Fill the PMWnds list box with our window and the
   // windows of the Program Manager.

   // First our own dialog box
   hwndT = GetDlgItem(hwnd, IDC_PMWNDS);
   ListBox_AddString(hwndT,
      __TEXT("---> This dialog box <---"));

   ListBox_SetItemData(hwndT, 0, hwnd);
   ListBox_SetCurSel(hwndT, 0);

   // Now the windows of the Program Manager
   g_hwndPM = FindWindow(__TEXT("PROGMAN"), NULL);
   WalkWindowTree(hwndT, g_hwndPM);

   return(TRUE);
}


//////////////////////////////////////////////////////////////


void Dlg_OnDestroy (HWND hwnd) {
   if (g_uTimerId != 0)
      KillTimer(hwnd, g_uTimerId);
}


//////////////////////////////////////////////////////////////


void Dlg_OnCommand (HWND hwnd, int id, HWND hwndCtl,
   UINT codeNotify) {

   HWND hwndT;
   RECT rc;

   switch (id) {

      case IDCANCEL:
         EndDialog(hwnd, 0);
         break;

      case IDC_FUNCSTART:
         g_dwEventTime = GetTickCount() + 1000 *
            GetDlgItemInt(hwnd, IDC_DELAY, NULL, FALSE);
         hwndT = GetDlgItem(hwnd, IDC_PMWNDS);
         g_hwndSubject = (HWND)
            ListBox_GetItemData(hwndT,
               ListBox_GetCurSel(hwndT));
         g_nEventId =
            ComboBox_GetCurSel(GetDlgItem(hwnd, IDC_WNDFUNC));
         SetWindowText(GetDlgItem(hwnd, IDC_EVENTPENDING),
            __TEXT("Pending"));
         break;

      case IDC_THREADATTACH:
         AttachThreadInput(
            GetWindowThreadProcessId(g_hwndPM, NULL),
            GetCurrentThreadId(), TRUE);
         break;

      case IDC_THREADDETACH:
         AttachThreadInput(
            GetWindowThreadProcessId(g_hwndPM, NULL),
            GetCurrentThreadId(), FALSE);
         break;

      case IDC_SETCLIPRECT:
         SetRect(&rc, 0, 0, 200, 200);
         ClipCursor(&rc);
         break;

      case IDC_REMOVECLIPRECT:
         ClipCursor(NULL);
         break;

      case IDC_HIDECURSOR:
         ShowCursor(FALSE);
         break;

      case IDC_SHOWCURSOR:
         ShowCursor(TRUE);
         break;

      case IDC_INFINITELOOP:
         SetCursor(LoadCursor(NULL, IDC_NO));
         for (;;)
             ;
         break;
   }
}


//////////////////////////////////////////////////////////////


BOOL Dlg_OnSetCursor (HWND hwnd, HWND hwndCursor,
   UINT codeHitTest, UINT msg) {

   SetCursor(LoadCursor(NULL, IDC_UPARROW));
   return(TRUE);
}


//////////////////////////////////////////////////////////////


void AddStr (HWND hwndLB, LPCTSTR szBuf) {
   int nIndex;

   do {
      nIndex = ListBox_AddString(hwndLB, szBuf);
      if (nIndex == LB_ERR)
         ListBox_DeleteString(hwndLB, 0);
   } while (nIndex == LB_ERR);

   ListBox_SetCurSel(hwndLB, nIndex);
}


//////////////////////////////////////////////////////////////


int Dlg_OnRButtonDown (HWND hwnd, BOOL fDoubleClick,
   int x, int y, UINT keyFlags) {

   TCHAR szBuf[100];
   _stprintf(szBuf,
      __TEXT("Capture=%-3s, Msg=RButtonDown, ")
      __TEXT("DblClk=%-3s, x=%5d, y=%5d"),
      (GetCapture() == NULL) ? __TEXT("No") : __TEXT("Yes"),
      fDoubleClick ? __TEXT("Yes") : __TEXT("No"), x, y);

   AddStr(GetDlgItem(hwnd, IDC_MOUSEMSGS), szBuf);
   if (!fDoubleClick) {
      SetCapture(hwnd);
   } else {
      ReleaseCapture();
   }
   return(0);
}


//////////////////////////////////////////////////////////////


int Dlg_OnRButtonUp (HWND hwnd, int x, int y, UINT keyFlags) {
   TCHAR szBuf[100];
   _stprintf(szBuf,
      __TEXT("Capture=%-3s, Msg=RButtonUp,   x=%5d, y=%5d"),
      (GetCapture() == NULL)
         ? __TEXT("No") : __TEXT("Yes"), x, y);

   AddStr(GetDlgItem(hwnd, IDC_MOUSEMSGS), szBuf);
   return(0);
}


/////////////////////////////////////////////////////////////


int Dlg_OnLButtonDown (HWND hwnd, BOOL fDoubleClick,
   int x, int y, UINT keyFlags) {

   TCHAR szBuf[100];
   _stprintf(szBuf,
      __TEXT("Capture=%-3s, Msg=LButtonDown, ")
      __TEXT("DblClk=%-3s, x=%5d, y=%5d"),
      (GetCapture() == NULL) ? __TEXT("No") : __TEXT("Yes"),
      fDoubleClick ? __TEXT("Yes") : __TEXT("No"), x, y);

   AddStr(GetDlgItem(hwnd, IDC_MOUSEMSGS), szBuf);
   return(0);
}


//////////////////////////////////////////////////////////////


void Dlg_OnLButtonUp (HWND hwnd, int x, int y, 
   UINT keyFlags) {
   TCHAR szBuf[100];

   _stprintf(szBuf,
      __TEXT("Capture=%-3s, Msg=LButtonUp,   x=%5d, y=%5d"),
      (GetCapture() == NULL)
         ? __TEXT("No") : __TEXT("Yes"), x, y);

   AddStr(GetDlgItem(hwnd, IDC_MOUSEMSGS), szBuf);
}


//////////////////////////////////////////////////////////////


void Dlg_OnMouseMove (HWND hwnd, int x, int y, 
   UINT keyFlags) {
   
   TCHAR szBuf[100];

   _stprintf(szBuf,
      __TEXT("Capture=%-3s, Msg=MouseMove,  x=%5d, y=%5d"),
         (GetCapture() == NULL)
         ? __TEXT("No") : __TEXT("Yes"), x, y);

   AddStr(GetDlgItem(hwnd, IDC_MOUSEMSGS), szBuf);
}


//////////////////////////////////////////////////////////////


void Dlg_OnTimer (HWND hwnd, UINT id) {
   TCHAR szBuf[100];
   RECT rc;
   HWND hwndT;

   CalcWndText(GetFocus(), szBuf, ARRAY_SIZE(szBuf));
   SetWindowText(GetDlgItem(hwnd, IDC_WNDFOCUS), szBuf);

   CalcWndText(GetCapture(), szBuf, ARRAY_SIZE(szBuf));
   SetWindowText(GetDlgItem(hwnd, IDC_WNDCAPTURE), szBuf);

   CalcWndText(GetActiveWindow(), szBuf, ARRAY_SIZE(szBuf));
   SetWindowText(GetDlgItem(hwnd, IDC_WNDACTIVE), szBuf);

   CalcWndText(GetForegroundWindow(), szBuf,
      ARRAY_SIZE(szBuf));
   SetWindowText(GetDlgItem(hwnd, IDC_WNDFOREGROUND), szBuf);

   GetClipCursor(&rc);
   _stprintf(szBuf,
      __TEXT("left=%d, top=%d, right=%d, bottom=%d"),
      rc.left, rc.top, rc.right, rc.bottom);
   SetWindowText(GetDlgItem(hwnd, IDC_CLIPCURSOR), szBuf);


   if ((g_dwEventTime == 0) ||
       (GetTickCount() < g_dwEventTime))
      return;

   switch (g_nEventId) {
      case 0:  // SetFocus
         g_hwndSubject = SetFocus(g_hwndSubject);
         break;

      case 1:  // SetActiveWindow
         g_hwndSubject = SetActiveWindow(g_hwndSubject);
         break;

      case 2:  // SetForegroundWindow
         hwndT = GetForegroundWindow();
         SetForegroundWindow(g_hwndSubject);
         g_hwndSubject = hwndT;
         break;

      case 3:  // BringWindowToTop
         BringWindowToTop(g_hwndSubject);
         break;

      case 4:  // SetWindowPos w/HWND_TOP
         SetWindowPos(g_hwndSubject, HWND_TOP, 0, 0, 0, 0,
            SWP_NOMOVE | SWP_NOSIZE);
         g_hwndSubject = (HWND) 1;
         break;

      case 5:  // SetWindowPos w/ HWND_BOTTOM
         SetWindowPos(g_hwndSubject, HWND_BOTTOM, 0, 0, 0, 0,
            SWP_NOMOVE | SWP_NOSIZE);
         g_hwndSubject = (HWND) 1;
         break;
   }

   if (g_hwndSubject == (HWND) 1) {
      SetWindowText(GetDlgItem(hwnd, IDC_PREVWND),
         __TEXT("Can't tell."));
   } else {
      CalcWndText(g_hwndSubject, szBuf, ARRAY_SIZE(szBuf));
      SetWindowText(GetDlgItem(hwnd, IDC_PREVWND), szBuf);
   }

   g_hwndSubject = NULL; g_nEventId = 0; g_dwEventTime = 0;
   SetWindowText(GetDlgItem(hwnd, IDC_EVENTPENDING),
      __TEXT("Executed"));
}


//////////////////////////////////////////////////////////////


BOOL CALLBACK Dlg_Proc (HWND hDlg, UINT uMsg,
   WPARAM wParam, LPARAM lParam) {

   BOOL fProcessed = TRUE;

   switch (uMsg) {
      HANDLE_MSG(hDlg, WM_INITDIALOG,     Dlg_OnInitDialog);
      HANDLE_MSG(hDlg, WM_DESTROY,        Dlg_OnDestroy);
      HANDLE_MSG(hDlg, WM_COMMAND,        Dlg_OnCommand);

      HANDLE_MSG(hDlg, WM_MOUSEMOVE,      Dlg_OnMouseMove);

      HANDLE_MSG(hDlg, WM_LBUTTONDOWN,    Dlg_OnLButtonDown);
      HANDLE_MSG(hDlg, WM_LBUTTONDBLCLK,  Dlg_OnLButtonDown);
      HANDLE_MSG(hDlg, WM_LBUTTONUP,      Dlg_OnLButtonUp);

      HANDLE_MSG(hDlg, WM_RBUTTONDOWN,    Dlg_OnRButtonDown);
      HANDLE_MSG(hDlg, WM_RBUTTONDBLCLK,  Dlg_OnRButtonDown);
      HANDLE_MSG(hDlg, WM_RBUTTONUP,      Dlg_OnRButtonUp);

      HANDLE_MSG(hDlg, WM_SETCURSOR,      Dlg_OnSetCursor);
      HANDLE_MSG(hDlg, WM_TIMER,          Dlg_OnTimer);

      default:
         fProcessed = FALSE;
         break;
   }
   return(fProcessed);
}


//////////////////////////////////////////////////////////////


int WINAPI WinMain (HINSTANCE hinstExe,
   HINSTANCE hinstPrev, LPSTR lpszCmdLine, int nCmdShow) {

   DialogBox(hinstExe, MAKEINTRESOURCE(IDD_LISLAB),
      NULL, Dlg_Proc);

   return(0);
}


//////////////////////// End Of File /////////////////////////
