/*************************************************************
Module name: MMFShare.C
Notices: Copyright (c) 1995 Jeffrey Richter
*************************************************************/


#include "..\AdvWin32.H"     /* See Appendix B for details. */
#include <windows.h>
#include <windowsx.h>

#pragma warning(disable: 4001)       /* Single-line comment */

#include "Resource.H"


//////////////////////////////////////////////////////////////


BOOL Dlg_OnInitDialog (HWND hwnd, HWND hwndFocus, 
   LPARAM lParam) {

    // Associate an icon with the dialog box.
    SetClassLong(hwnd, GCL_HICON, (LONG)
      LoadIcon((HINSTANCE) GetWindowLong(hwnd, GWL_HINSTANCE),
      __TEXT("MMFShare")));

   // Initialize the edit control with some test data.
   Edit_SetText(GetDlgItem(hwnd, IDC_DATA), 
      __TEXT("Some test data"));

   // Disable the Close button because the file can't
   // be closed if it was never created or opened.
   Button_Enable(GetDlgItem(hwnd, IDC_CLOSEFILE), FALSE);
   return(TRUE);
}


//////////////////////////////////////////////////////////////


void Dlg_OnCommand (HWND hwnd, int id, HWND hwndCtl, 
   UINT codeNotify) {
   
   // Handle of the open memory-mapped file
   static HANDLE s_hFileMap = NULL;
   HANDLE hFileMapT;

   switch (id) {
      case IDC_CREATEFILE:
         if (codeNotify != BN_CLICKED)
            break;

         // Create an in-memory memory-mapped file that 
         // contains the contents of the edit control. The 
         // file is 4 KB at most and is named MMFSharedData.
         s_hFileMap = CreateFileMapping((HANDLE) 0xFFFFFFFF, 
            NULL, PAGE_READWRITE, 0, 4 * 1024, 
            __TEXT("MMFSharedData"));

         if (s_hFileMap != NULL) {

            if (GetLastError() == ERROR_ALREADY_EXISTS) {
               MessageBox(hwnd,
                  __TEXT("Mapping already exists - ")
                  __TEXT("not created."),
                  NULL, MB_OK);
               CloseHandle(s_hFileMap);

            } else {

               // File mapping created successfully.

               // Map a view of the file 
               // into the address space.
               LPVOID lpView = MapViewOfFile(s_hFileMap,
                  FILE_MAP_READ | FILE_MAP_WRITE, 0, 0, 0);

               if ((BYTE *) lpView != NULL) {
                  // View mapped successfully; put contents 
                  // of edit control into the memory-mapped
                  // file.
                  Edit_GetText(GetDlgItem(hwnd, IDC_DATA),
                     (LPTSTR) lpView, 4 * 1024);

                  // Unmap the view. This protects the
                  // data from wayward pointers.
                  UnmapViewOfFile((LPVOID) lpView);

                  // The user can't create 
                  // another file right now.
                  Button_Enable(hwndCtl, FALSE);

                  // The user closed the file.
                  Button_Enable(GetDlgItem(hwnd,
                     IDC_CLOSEFILE), TRUE);

               } else {
                  MessageBox(hwnd, 
                     __TEXT("Can't map view of file."),
                     NULL, MB_OK);
               }
            }

         } else {
            MessageBox(hwnd, 
               __TEXT("Can't create file mapping."),
               NULL, MB_OK);
         }
         break;

      case IDC_CLOSEFILE:
         if (codeNotify != BN_CLICKED)
            break;

         if (CloseHandle(s_hFileMap)) {
            // User closed the file. A new file can be
            // created, but the new file can't be closed.
            Button_Enable(GetDlgItem(hwnd, IDC_CREATEFILE), 
               TRUE);
            Button_Enable(hwndCtl, FALSE);
         }
         break;

      case IDC_OPENFILE:
         if (codeNotify != BN_CLICKED)
            break;

         // See if a memory-mapped file named
         // MMFSharedData already exists.
         hFileMapT = OpenFileMapping(
            FILE_MAP_READ | FILE_MAP_WRITE,
            FALSE, __TEXT("MMFSharedData"));

         if (hFileMapT != NULL) {
            // Memory-mapped file does exist.  Map a view
            // of it into the process's address space.
            LPVOID lpView = MapViewOfFile(hFileMapT,
               FILE_MAP_READ | FILE_MAP_WRITE, 0, 0, 0);

            if ((BYTE *) lpView != NULL) {

               // Put the contents of the 
               // file into the edit control.
               Edit_SetText(GetDlgItem(hwnd, IDC_DATA),
                  (LPTSTR) lpView);
               UnmapViewOfFile((LPVOID) lpView);

            } else {
               MessageBox(hwnd, 
                  __TEXT("Can't map view."), NULL, MB_OK);
            }

            CloseHandle(hFileMapT);

         } else {
            MessageBox(hwnd, 
               __TEXT("Can't open mapping."), NULL, MB_OK);
         }
         break;

      case IDCANCEL:
         EndDialog(hwnd, id);
         break;
   }
}


//////////////////////////////////////////////////////////////


BOOL CALLBACK Dlg_Proc (HWND hDlg, UINT uMsg, 
   WPARAM wParam, LPARAM lParam) {
   
   BOOL fProcessed = TRUE;

   switch (uMsg) {
      HANDLE_MSG(hDlg, WM_INITDIALOG, Dlg_OnInitDialog);
      HANDLE_MSG(hDlg, WM_COMMAND, Dlg_OnCommand);

      default:
         fProcessed = FALSE;
         break;
   }
   return(fProcessed);
}


//////////////////////////////////////////////////////////////


int WINAPI WinMain (HINSTANCE hinstExe,
   HINSTANCE hinstPrev, LPSTR lpszCmdLine, int nCmdShow) {

   DialogBox(hinstExe, MAKEINTRESOURCE(IDD_MMFSHARE), 
      NULL, Dlg_Proc);

   return(0);
}


/////////////////////// End Of File ///////////////////////
