/************************************************************
Module name: PMRstSub.C
Notices: Copyright (c) 1995 Jeffrey Richter
************************************************************/


#include "..\AdvWin32.H"    /* See Appendix B for details. */
#include <windows.h>
#include <windowsx.h>

#pragma warning(disable: 4001)      /* Single-line comment */

#include "PMRstSub.RH"


#define _PMRSTSUBLIB_
#include "PMRstSub.H"


/////////////////////////////////////////////////////////////


// Forward references
LRESULT WINAPI GetMsgProc (int nCode, WPARAM wParam,
   LPARAM lParam);

LRESULT WINAPI PMSubclass (HWND hwnd, UINT uMsg,
   WPARAM wParam, LPARAM lParam);

BOOL WINAPI AnyAppsRunning (HWND hwnd, LPARAM lParam);


/////////////////////////////////////////////////////////////


// Instruct the compiler to put the g_dwThreadIdPMRestore and
// the g_hwndPM data variable in their own data section called
// Shared. We then instruct the linker that we want to
// share the data in this section with all instances of this
// application.
#pragma data_seg("Shared")

DWORD g_dwThreadIdPMRestore = 0;
HWND  g_hwndPM = NULL;

#pragma data_seg()


/////////////////////////////////////////////////////////////


// Instruct the linker to make the Shared section 
// readable, writable, and shared.
#pragma comment(lib, "msvcrt " "-section:Shared,rws")


/////////////////////////////////////////////////////////////


// Nonshared variables
PMRSTSUBAPI HHOOK g_hHook = NULL;
WNDPROC   g_wpOrigPMProc = NULL;
HINSTANCE g_hinstDll = NULL;


/////////////////////////////////////////////////////////////


BOOL WINAPI DllMain (HINSTANCE hinstDll, DWORD fdwReason,
   LPVOID lpvReserved) {

   switch (fdwReason) {

      case DLL_PROCESS_ATTACH:
         // DLL is attaching to the address space
         // of the current process.
         g_hinstDll = hinstDll;
         break;

      case DLL_THREAD_ATTACH:
         // A new thread is being created
         // in the current process.
         break;

      case DLL_THREAD_DETACH:
         // A thread is exiting cleanly.
         break;

      case DLL_PROCESS_DETACH:
         // The calling process is detaching the
         // DLL from its address space.
         break;
   }
   return(TRUE);
}


/////////////////////////////////////////////////////////////


// Menu IDs from Program Manager's menu
#define IDM_PMRESTOREABOUT       (4444)
#define IDM_PMRESTOREREMOVE      (4445)

PMRSTSUBAPI BOOL SubclassProgManFrame (DWORD
   dwThreadIdPMRestore) {

   HMENU hmenu;

   // Find the window handle of the Program Manager. Do not
   // specify a caption because the Program Manager's caption
   // changes depending on whether a group is maximized or not.
   g_hwndPM = FindWindow(__TEXT("PROGMAN"), NULL);

   if (!IsWindow(g_hwndPM)) {
      // If the Program Manager cannot be found,
      // we must terminate.
      MessageBox(NULL,
         __TEXT("Cannot find the Program Manager."),
         NULL, MB_OK);
      return(FALSE);
   }

   // First we must install a systemwide WH_GETMESSAGE hook.
   g_hHook = SetWindowsHookEx(WH_GETMESSAGE, GetMsgProc,
      g_hinstDll, GetWindowThreadProcessId(g_hwndPM, NULL));

   // The hook cannot be installed. (Maybe there is greater
   // security in this environment.)
   if (g_hHook == NULL)
      return(FALSE);

   // The hook was installed successfully; force a
   // benign message to the window so that the hook
   // function gets called.
   PostMessage(g_hwndPM, WM_NULL, 0, 0);


   // Get the menu handle to the Program Manager's Options menu.
   hmenu = GetMenu(g_hwndPM);

   AppendMenu(hmenu, MF_ENABLED | MF_STRING,
      IDM_PMRESTOREABOUT, __TEXT("A&bout PM Restore..."));
   AppendMenu(hmenu, MF_ENABLED | MF_STRING,
      IDM_PMRESTOREREMOVE, __TEXT("&Remove PM Restore"));

   DrawMenuBar(g_hwndPM);      // Update the new menu bar.

   // The window in the other process is subclassed.
   g_dwThreadIdPMRestore = dwThreadIdPMRestore;
   return(TRUE);
}


/////////////////////////////////////////////////////////////


LRESULT WINAPI GetMsgProc (int nCode, WPARAM wParam,
   LPARAM lParam) {

   static BOOL fPMSubclassed = FALSE;

   if (!fPMSubclassed && (nCode == HC_ACTION)  &&
      (wParam == PM_REMOVE) &&
      (((MSG *) lParam)->hwnd == g_hwndPM) &&
      (((MSG *) lParam)->message == WM_NULL)) {
      // If we have not yet subclassed the Program Manager
      // and it is retrieving a message
      // and the window handle identifies the Program Manager 
      // and the message is a WM_NULL message

      // This DLL is now mapped into the Program Manager's
      // address space.  Time to subclass the Program Manager.
      g_wpOrigPMProc = SubclassWindow(g_hwndPM, PMSubclass);

      // Remind ourselves that we have subclassed the
      // Program Manager so that we don't do it again if we
      // ever get another WM_NULL message.
      fPMSubclassed = TRUE;
   }

   return(CallNextHookEx(g_hHook, nCode, wParam, lParam));
}


/////////////////////////////////////////////////////////////


int PM_OnActivateApp (HWND hwnd, BOOL fActivate,
   DWORD dwThreadId) {

   BOOL fAnyWindowsUp;

   // The Program Manager is being either activated 
   // or deactivated.
   if (!fActivate)
      return(0);     // PROGMAN being deactivated.

   if (!IsIconic(hwnd))
      return(0);     // PROGMAN isn't an icon.

   // Program Manager is being made active and is an icon.
   // Check whether any other applications are running.
   fAnyWindowsUp = (EnumWindows(AnyAppsRunning, 0) == 0);

   // If the enumeration was stopped prematurely, there must
   // be at least one other application running.
   if (fAnyWindowsUp)
      return(0);

   // No other apps running; restore PROGMAN to "open" state.
   ShowWindow(hwnd, SW_RESTORE);
   return(0);
}


/////////////////////////////////////////////////////////////


// Function to process About box
BOOL WINAPI AboutProc (HWND hDlg, UINT uMsg,
   WPARAM wParam, LPARAM lParam) {

   BOOL fProcessed = TRUE;

   switch (uMsg) {
      case WM_INITDIALOG:
         break;

      case WM_COMMAND:
         switch (GET_WM_COMMAND_ID(wParam, lParam)) {
            case IDOK:
            case IDCANCEL:
               if (GET_WM_COMMAND_CMD(wParam, lParam) ==
                  BN_CLICKED) {
                  EndDialog(hDlg,
                     GET_WM_COMMAND_ID(wParam, lParam));
               }
               break;

            default:
               break;
         }
         break;

      default:
         fProcessed = FALSE;
         break;
   }
   return(fProcessed);
}


/////////////////////////////////////////////////////////////


void PM_OnCommand (HWND hwnd, int id, HWND hwndCtl,
   UINT codeNotify) {

   HMENU hmenu;

   switch (id) {

      case IDM_PMRESTOREABOUT:
         // The About PM Restore menu option
         // that we added was chosen.
         DialogBox(g_hinstDll, MAKEINTRESOURCE(IDD_ABOUT),
            hwnd, AboutProc);
         break;

      case IDM_PMRESTOREREMOVE:
         // Stop window subclassing by putting back the
         // address of the original window procedure.
         (void) SubclassWindow(hwnd, g_wpOrigPMProc);

         // Get the menu handle to the Program Manager's
         // Options menu.
         hmenu = GetMenu(hwnd);
         RemoveMenu(hmenu, IDM_PMRESTOREABOUT, MF_BYCOMMAND);
         RemoveMenu(hmenu, IDM_PMRESTOREREMOVE, MF_BYCOMMAND);
         DrawMenuBar(hwnd);      // Update the new menu bar.

         // Post WM_QUIT to our task to remove it from memory.
         PostThreadMessage(g_dwThreadIdPMRestore, WM_QUIT,
            0, 0);
         break;

      default: // Pass other WM_COMMANDs to original WndProc.
         break;
   }
}


/////////////////////////////////////////////////////////////


// Subclass function for the Program Manager. Any message for
// the Program Manager window comes here before reaching the
// original window function.
LRESULT WINAPI PMSubclass (HWND hwnd, UINT uMsg,
   WPARAM wParam, LPARAM lParam) {

   switch (uMsg) {
      case WM_ACTIVATEAPP:
         HANDLE_WM_ACTIVATEAPP(hwnd, wParam,
            lParam, PM_OnActivateApp);
         break;

      case WM_COMMAND:
         HANDLE_WM_COMMAND(hwnd, wParam, lParam, 
            PM_OnCommand);
         break;

      default: // Pass other messages to original procedure.
         break;
   }

   // Call original window procedure and return the result to
   // whoever sent this message to the Program Manager.
   return(CallWindowProc(g_wpOrigPMProc, hwnd,
      uMsg, wParam, lParam));
}


/////////////////////////////////////////////////////////////


// This callback function determines whether any 
// windows exist that should stop us from restoring
// the Program Manager.
BOOL WINAPI AnyAppsRunning (HWND hwnd, LPARAM lParam) {

   // If the window is the Windows desktop,
   // continue enumeration.
   if (hwnd == GetDesktopWindow())
      return(1);

   // If the window is invisible (hidden), 
   // continue enumeration.
   if (!IsWindowVisible(hwnd))
      return(1);

   // If the window was created by PROGMAN,
   // continue enumeration.
   if (GetWindowThreadProcessId(g_hwndPM, NULL) ==
      GetWindowThreadProcessId(hwnd, NULL))
      return(1);

   // For any other type of window, stop enumeration.
   return(0);
}


/////////////////////// End Of File ///////////////////////
