/***********************************************************
Module name: SprMrkt.C
Notices: Copyright (c) 1995 Jeffrey Richter
***********************************************************/


#include "..\AdvWin32.H"   /* See Appendix B for details. */
#include <windows.h>
#include <windowsx.h>

#pragma warning(disable: 4001)     /* Single-line comment */

#include <tchar.h>
#include <stdio.h>
#include <stdlib.h>            // For random number stuff
#include <string.h>
#include <stdarg.h>
#include <process.h>           // For _beginthreadex
#include "Resource.H"


// This is the correction to a bug in windowsx.h:
#undef FORWARD_WM_HSCROLL
#define FORWARD_WM_HSCROLL(hwnd, hwndCtl, code, pos, fn) \
    (void)(fn)((hwnd), WM_HSCROLL, \
       MAKEWPARAM((UINT)(code),(UINT)(pos)), \
       (LPARAM)(UINT)(hwndCtl))


///////////////////////////////////////////////////////////


// Forward references to the supermarket
// and shopper thread functions.
DWORD WINAPI ThreadSuperMarket (LPVOID lpvParam);
DWORD WINAPI ThreadShopper (LPVOID lpvParam);


//////////////////////////////////////////////////////////////


// Global variables
HWND g_hwndLB = NULL;   // List box for shopper events

// User-settable simulation parameters.
int g_nMaxOccupancy;
int g_nTimeOpen;
int g_nCheckoutCounters;
int g_nMaxDelayBetweenShopperCreation;
int g_nMaxWaitToGetInMarket;
int g_nMaxTimeShopping;
int g_nMaxWaitForDeliCntr;
int g_nMaxTimeSpentAtDeli;
int g_nMaxTimeAtCheckout;

// Synchronization objects used to control the simulation
HANDLE g_hSemEntrance;
HANDLE g_hMtxDeliCntr;
HANDLE g_hSemCheckout;


//////////////////////////////////////////////////////////////


// This function constructs a string using the format string
// passed and the variable number of arguments and adds the
// string to the shopper events list box identified by the
// global g_hwndLB variable.
void AddStr (LPCTSTR szFmt, ...) {
   TCHAR szBuf[150];
   int nIndex;
   va_list va_params;

   // Make va_params point to the first argument after szFmt.
   va_start(va_params, szFmt);

   // Build the string to be displayed.
   _vstprintf(szBuf, szFmt, va_params);
   do {
      // Add the string to the end of the list box.
      nIndex = ListBox_AddString(g_hwndLB, szBuf);

      // If the list box is full, delete the first item in it.
      if (nIndex == LB_ERR)
         ListBox_DeleteString(g_hwndLB, 0);

   } while (nIndex == LB_ERR);

   // Select the newly added item.
   ListBox_SetCurSel(g_hwndLB, nIndex);

   // Indicate that we are done referencing
   // the variable arguments.
   va_end(va_params);
}


//////////////////////////////////////////////////////////////


// This function returns a random number
// between 0 and nMaxValue, inclusive.
int Random (int nMaxValue) {
   return(((2 * rand() * nMaxValue + RAND_MAX) /
      RAND_MAX - 1) / 2);
}


//////////////////////////////////////////////////////////////


BOOL Dlg_OnInitDialog (HWND hwnd, HWND hwndFocus,
   LPARAM lParam) {

   HWND hwndSB;

   // Associate an icon with the dialog box.
   SetClassLong(hwnd, GCL_HICON, (LONG)
      LoadIcon((HINSTANCE) GetWindowLong(hwnd, GWL_HINSTANCE),
      __TEXT("SprMrkt")));

   // Save the window handle to the shopper events list box
   // in a global variable so that the AddStr function has
   // access to it.
   g_hwndLB = GetDlgItem(hwnd, IDC_SHOPPEREVENTS);

   // Set the scroll bar range and default positions for all of
   // the simulation parameters.
   hwndSB = GetDlgItem(hwnd, IDC_MAXOCCUPANCY);
   ScrollBar_SetRange(hwndSB, 0, 500, TRUE);

   // Set the initial value of the scroll bar.
   FORWARD_WM_HSCROLL(hwnd, hwndSB, SB_THUMBTRACK,
      30, SendMessage);


   hwndSB = GetDlgItem(hwnd, IDC_TIMEOPEN);
   ScrollBar_SetRange(hwndSB, 0, 5000, TRUE);
   FORWARD_WM_HSCROLL(hwnd, hwndSB, SB_THUMBTRACK,
      5000, SendMessage);

   hwndSB = GetDlgItem(hwnd, IDC_NUMCOUNTERS);
   ScrollBar_SetRange(hwndSB, 0, 30, TRUE);
   FORWARD_WM_HSCROLL(hwnd, hwndSB, SB_THUMBTRACK,
      5, SendMessage);

   hwndSB = GetDlgItem(hwnd, IDC_SHOPPERCREATIONDELAY);
   ScrollBar_SetRange(hwndSB, 0, 1000, TRUE);
   FORWARD_WM_HSCROLL(hwnd, hwndSB, SB_THUMBTRACK,
      300, SendMessage);

   hwndSB = GetDlgItem(hwnd, IDC_DELAYTOGETIN);
   ScrollBar_SetRange(hwndSB, 0, 100, TRUE);
   FORWARD_WM_HSCROLL(hwnd, hwndSB, SB_THUMBTRACK,
      20, SendMessage);

   hwndSB = GetDlgItem(hwnd, IDC_TIMETOSHOP);
   ScrollBar_SetRange(hwndSB, 0, 100, TRUE);
   FORWARD_WM_HSCROLL(hwnd, hwndSB, SB_THUMBTRACK,
      80, SendMessage);

   hwndSB = GetDlgItem(hwnd, IDC_WAITDELICNTR);
   ScrollBar_SetRange(hwndSB, 0, 100, TRUE);
   FORWARD_WM_HSCROLL(hwnd, hwndSB, SB_THUMBTRACK,
      20, SendMessage);

   hwndSB = GetDlgItem(hwnd, IDC_TIMEATDELICNTR);
   ScrollBar_SetRange(hwndSB, 0, 100, TRUE);
   FORWARD_WM_HSCROLL(hwnd, hwndSB, SB_THUMBTRACK,
      70, SendMessage);

   hwndSB = GetDlgItem(hwnd, IDC_TIMEATCHECKOUT);
   ScrollBar_SetRange(hwndSB, 0, 100, TRUE);
   FORWARD_WM_HSCROLL(hwnd, hwndSB, SB_THUMBTRACK,
      60, SendMessage);

   return(TRUE);
}


//////////////////////////////////////////////////////////////


void Dlg_OnHScroll(HWND hwnd, HWND hwndCtl,
   UINT code, int pos) {

   TCHAR szBuf[10];
   int nPosCrnt, nPosMin, nPosMax;

   // Get the current position and the legal range for the
   // scroll bar that the user is changing.
   nPosCrnt = ScrollBar_GetPos(hwndCtl);
   ScrollBar_GetRange(hwndCtl, &nPosMin, &nPosMax);

   switch (code) {
      case SB_LINELEFT:
         nPosCrnt--;
         break;

      case SB_LINERIGHT:
         nPosCrnt++;
         break;

      case SB_PAGELEFT:
         nPosCrnt -= (nPosMax - nPosMin + 1) / 10;
         break;

      case SB_PAGERIGHT:
         nPosCrnt += (nPosMax - nPosMin + 1) / 10;
         break;

      case SB_THUMBTRACK:
         nPosCrnt = pos;
         break;

      case SB_LEFT:
         nPosCrnt = nPosMin;
         break;

      case SB_RIGHT:
         nPosCrnt = nPosMax;
         break;
   }

   // Make sure that the new scroll bar position
   // is within the legal range.
   if (nPosCrnt < nPosMin)
      nPosCrnt = nPosMin;

   if (nPosCrnt > nPosMax)
      nPosCrnt = nPosMax;

   // Set the new scroll bar position.
   ScrollBar_SetPos(hwndCtl, nPosCrnt, TRUE);

   // Change the value displayed in the text box to
   // reflect the value in the scroll bar.
   _stprintf(szBuf, __TEXT("%d"), nPosCrnt);
   SetWindowText(GetPrevSibling(hwndCtl), szBuf);
}


//////////////////////////////////////////////////////////////


void Dlg_OnCommand (HWND hwnd, int id, HWND hwndCtl,
   UINT codeNotify) {

   DWORD dwThreadId;
   HANDLE hThread;

   switch (id) {
      case IDOK:
         // Load the scroll bar settings into the global
         // variables so that they can be used
         // by the simulation.
         g_nMaxOccupancy = ScrollBar_GetPos(
            GetDlgItem(hwnd, IDC_MAXOCCUPANCY));

         g_nTimeOpen = ScrollBar_GetPos(
            GetDlgItem(hwnd, IDC_TIMEOPEN));

         g_nCheckoutCounters = ScrollBar_GetPos(
            GetDlgItem(hwnd, IDC_NUMCOUNTERS));

         g_nMaxDelayBetweenShopperCreation = ScrollBar_GetPos(
            GetDlgItem(hwnd, IDC_SHOPPERCREATIONDELAY));

         g_nMaxWaitToGetInMarket = ScrollBar_GetPos(
            GetDlgItem(hwnd, IDC_DELAYTOGETIN));

         g_nMaxTimeShopping = ScrollBar_GetPos(
            GetDlgItem(hwnd, IDC_TIMETOSHOP));

         g_nMaxWaitForDeliCntr = ScrollBar_GetPos(
            GetDlgItem(hwnd, IDC_WAITDELICNTR));

         g_nMaxTimeSpentAtDeli = ScrollBar_GetPos(
            GetDlgItem(hwnd, IDC_TIMEATDELICNTR));

         g_nMaxTimeAtCheckout = ScrollBar_GetPos(
            GetDlgItem(hwnd, IDC_TIMEATCHECKOUT));


         // Clear out everything in the list box.
         ListBox_ResetContent(
            GetDlgItem(hwnd, IDC_SHOPPEREVENTS));

         // Disable the Open For Business button
         // while simulation is in progress.

         EnableWindow(hwndCtl, FALSE);

         if (NULL == GetFocus()) {
            SetFocus(GetDlgItem(hwnd, IDC_MAXOCCUPANCY));
         }


         // The system overhead will cause the results
         // of the simulation to be skewed. To help
         // minimize this effect, we boost the priority class
         // of this process.
         SetPriorityClass(GetCurrentProcess(),
            HIGH_PRIORITY_CLASS);

         // Create the thread representing the supermarket.
         hThread = BEGINTHREADEX(
            NULL,              // Security attributes
            0,                 // Stack
            ThreadSuperMarket, // Thread function
            (LPVOID) hwnd,     // Thread function parameter
            0,                 // Flags
            &dwThreadId);      // Thread ID    
            
         // Since we are not interested in manipulating the
         // thread object from this function, we can close
         // our handle to it.
         CloseHandle(hThread);
         break;

      case IDCANCEL:
         EndDialog(hwnd, id);
         break;
   }
}


//////////////////////////////////////////////////////////////


BOOL CALLBACK Dlg_Proc (HWND hDlg, UINT uMsg,
   WPARAM wParam, LPARAM lParam) {

   BOOL fProcessed = TRUE;

   switch (uMsg) {
      HANDLE_MSG(hDlg, WM_INITDIALOG,  Dlg_OnInitDialog);
      HANDLE_MSG(hDlg, WM_COMMAND,  Dlg_OnCommand);
      HANDLE_MSG(hDlg, WM_HSCROLL,  Dlg_OnHScroll);

      case WM_USER:
         // This message is sent by the SuperMarketThread function
         // to notify us that the simulation has completed.

         // Return the priority class of the simulation
         // back to normal.
         SetPriorityClass(GetCurrentProcess(),
            NORMAL_PRIORITY_CLASS);

         // Enable the Open For Business button so that
         // the user can run the simulation again with
         // new parameters.
         EnableWindow(GetDlgItem(hDlg, IDOK), TRUE);
         break;

      default:
         fProcessed = FALSE;
         break;
   }
   return(fProcessed);
}


//////////////////////////////////////////////////////////////


int WINAPI WinMain (HINSTANCE hinstExe,
   HINSTANCE hinstPrev, LPSTR lpszCmdLine, int nCmdShow) {

   DialogBox(hinstExe, MAKEINTRESOURCE(IDD_SPRMRKT),
      NULL, Dlg_Proc);

   return(0);
}


//////////////////////////////////////////////////////////////


DWORD WINAPI ThreadSuperMarket (LPVOID lpvParam) {
   DWORD dwCloseTime;
   HANDLE hThread;
   DWORD dwThreadId;
   int nShopperNum = 0, nMaxOccupancy;

   g_hSemEntrance = CreateSemaphore(
      NULL,                // Security attributes
      0,                   // Initial lock count
      g_nMaxOccupancy,     // Maximum people allowed in store
      NULL);               // Do not name the semaphore.

   g_hMtxDeliCntr = CreateMutex(
      NULL,                // Security attributes
      FALSE,               // Initially no one is at the deli.
      NULL);               // Do not name the mutex.

   g_hSemCheckout = CreateSemaphore(
      NULL,                // Security attributes
      g_nCheckoutCounters, // All counters are free.
      g_nCheckoutCounters, // The number of counters at the store.
      NULL);               // No name for the semaphore


   // Open the store to the shoppers.
   AddStr(__TEXT("---> Opening the supermarket to shoppers."));
   ReleaseSemaphore(g_hSemEntrance, g_nMaxOccupancy, NULL);

   // Get the time at which the store should
   // stop creating shoppers.
   dwCloseTime = GetTickCount() + g_nTimeOpen;

   // Continue loop until the store closes.
   while (GetTickCount() < dwCloseTime) {

      // Create the thread representing a shopper.
      hThread = BEGINTHREADEX(
         NULL,                   // Security attributes.
         0,                      // Stack
         ThreadShopper,          // Thread function
         (LPVOID) ++nShopperNum, // Shopper number as lpvParam
         0,                      // Flags
         &dwThreadId);           // Thread ID

      // Since we are not interested in manipulating the
      // thread object from this function, we can close
      // our handle to it.
      CloseHandle(hThread);

      // Wait until another shopper comes to the supermarket.
      Sleep(Random(g_nMaxDelayBetweenShopperCreation));
   }

   // The supermarket wants to close;
   // wait for all of the shoppers to leave.
   AddStr(__TEXT("---> Waiting for shoppers to check out ")
      __TEXT("so store can close."));

   nMaxOccupancy = 1;
   for (; nMaxOccupancy <= g_nMaxOccupancy; nMaxOccupancy++) {
      WaitForSingleObject(g_hSemEntrance, INFINITE);
      AddStr(__TEXT("---> %d shoppers NOT in store."),
      nMaxOccupancy);
   }

   AddStr(__TEXT("---> Store closed--end of simulation."));

   // Everybody has left the market--end of simulation.
   CloseHandle(g_hSemCheckout);
   CloseHandle(g_hMtxDeliCntr);
   CloseHandle(g_hSemEntrance);

   // Notify the GUI thread that the simulation has completed.
   // The window handle of the GUI thread's dialog box was
   // passed in the lpvParam parameter to this thread when
   // it was created.
   SendMessage((HWND) lpvParam, WM_USER, 0, 0);

   return(0);
}


//////////////////////////////////////////////////////////////


DWORD WINAPI ThreadShopper (LPVOID lpvParam) {
   int nShopperNum = (int) lpvParam;
   DWORD dwResult;
   int nDuration;

   // Wait till the shopper can enter the supermarket.
   nDuration = Random(g_nMaxWaitToGetInMarket);
   AddStr(__TEXT("%04lu: Waiting to get in store (%lu)."),
      nShopperNum, nDuration);

   dwResult = WaitForSingleObject(g_hSemEntrance, nDuration);
   if (dwResult == WAIT_TIMEOUT) {
      // The shopper got tired of
      // waiting to be let in and left.
      AddStr(__TEXT("%04lu: Tired of waiting; went home."),
      nShopperNum);
      return(0);
   }


   // Shopper entered the supermarket.  Time to go shopping.
   nDuration = Random(g_nMaxTimeShopping);
   AddStr(__TEXT("%04lu: In supermarket, shopping for %lu."),
      nShopperNum, nDuration);
   Sleep(nDuration);


   // Done with initial shopping. Shopper has a one-in-three
   // chance of going to the deli counter.
   if (Random(2) == 0) {

      // Shopper going to deli counter.
      nDuration = Random(g_nMaxWaitForDeliCntr);
      AddStr(
         __TEXT("%04lu: Waiting for service at ")
         __TEXT("Deli Counter (%lu)."),
         nShopperNum, nDuration);
      dwResult =
         WaitForSingleObject(g_hMtxDeliCntr, nDuration);

      if (dwResult == 0) {
         // Got attention at deli; order stuff.
         nDuration = Random(g_nMaxTimeSpentAtDeli);

         AddStr(__TEXT("%04lu: Being served at Deli (%lu)."),
            nShopperNum, nDuration);
         Sleep(nDuration);

         // Leave the deli counter.
         ReleaseMutex(g_hMtxDeliCntr);

      } else {
         // Tired of waiting at deli counter,
         // leave and continue shopping.
         AddStr(__TEXT("%04lu: Tired of waiting at Deli."),
            nShopperNum);
      }

   } else {
      AddStr(__TEXT("%04lu: Not going to the Deli counter."),
         nShopperNum);
   }


   // Waiting for a checkout counter.
   AddStr(
      __TEXT("%04lu: Waiting for an empty checkout counter."),
      nShopperNum);
   WaitForSingleObject(g_hSemCheckout, INFINITE);

   // Checking out.
   nDuration = Random(g_nMaxTimeAtCheckout);
   AddStr(__TEXT("%04lu: Checking out (%lu)."),
      nShopperNum, nDuration);
   Sleep(nDuration);

   // Leaving the checkout counter.
   AddStr(__TEXT("%04lu: Leaving checkout counter."),
      nShopperNum);
   ReleaseSemaphore(g_hSemCheckout, 1, NULL);

   // Leaving the store.
   AddStr(__TEXT("%04lu: Left the supermarket."),
      nShopperNum);
   ReleaseSemaphore(g_hSemEntrance, 1, NULL);

   // Shopper shopped till he/she dropped.  Shopper dead.
   return(0);
}


//////////////////////// End Of File /////////////////////////
