/***********************************************************
Module name: SysInfo.C
Notices: Copyright (c) 1995 Jeffrey Richter
***********************************************************/


#include "..\AdvWin32.H"   /* See Appendix B for details. */
#include <windows.h>
#include <windowsx.h>

#pragma warning(disable: 4001)     /* Single-line comment */

#include <tchar.h>
#include <stdio.h>
#include "Resource.H"


///////////////////////////////////////////////////////////


typedef struct {
   const DWORD dwValue; 
   LPCTSTR szText;
} LONGDATA;


LONGDATA CPUFlags[] = {
   { PROCESSOR_INTEL_386,     __TEXT("Intel 386")       },
   { PROCESSOR_INTEL_486,     __TEXT("Intel 486")       },
   { PROCESSOR_INTEL_PENTIUM, __TEXT("Intel Pentium")   },
   { PROCESSOR_INTEL_860,     __TEXT("Intel 860")       },
   { PROCESSOR_MIPS_R2000,    __TEXT("MIPS R2000")      },
   { PROCESSOR_MIPS_R3000,    __TEXT("MIPS R3000")      },
   { PROCESSOR_MIPS_R4000,    __TEXT("MIPS R4000")      },
   { PROCESSOR_ALPHA_21064,   __TEXT("DEC Alpha 21064") },
#ifdef PROCESSOR_PPC_601
   { PROCESSOR_PPC_601,       __TEXT("PowerPC 601")     },
   { PROCESSOR_PPC_603,       __TEXT("PowerPC 603")     },
   { PROCESSOR_PPC_604,       __TEXT("PowerPC 604")     },
   { PROCESSOR_PPC_620,       __TEXT("PowerPC 620")     },
#endif
   { 0, NULL }
};


///////////////////////////////////////////////////////////


LPCTSTR GetFlagStr (DWORD dwFlag, LONGDATA FlagList[],
   LPTSTR pszBuf) {

   int x;

   for (x = 0; FlagList[x].dwValue != 0; x++) {
      if (FlagList[x].dwValue == dwFlag)
         return(FlagList[x].szText);
   }

   _stprintf(pszBuf, __TEXT("Unknown (%d)"), dwFlag);
   return(pszBuf);
}


///////////////////////////////////////////////////////////


// This function accepts a number and converts it to a
// string, inserting commas where appropriate.
LPTSTR BigNumToString (LONG lNum, LPTSTR szBuf) {
   WORD wNumDigits = 0, wNumChars = 0;

   do {
      // Put the last digit of the string
      // in the character buffer.
      szBuf[wNumChars++] = (TCHAR) (lNum % 10 + __TEXT('0'));

      // Increment the number of digits
      // that we put in the string.
      wNumDigits++;

      // For every three digits put in
      // the string, add a comma (,).
      if (wNumDigits % 3 == 0)
         szBuf[wNumChars++] = __TEXT(',');

      // Divide the number by 10, and repeat the process.
      lNum /= 10;

      // Continue adding digits to the
      // string until the number is zero.
   } while (lNum != 0);

   // If the last character added to
   // the string was a comma, truncate it.
   if (szBuf[wNumChars - 1] == __TEXT(','))
      szBuf[wNumChars - 1] = 0;

   // Ensure that the string is zero-terminated.
   szBuf[wNumChars] = 0;

   // We added all the characters to the string in
   // reverse order. We must reverse the contents
   // of the string.
   _tcsrev(szBuf);

   // Returns the address of the string.  This is the same
   // value that was passed to us initially.  Returning it
   // here makes it easier for the calling function to
   // use the string.
   return(szBuf);
}


///////////////////////////////////////////////////////////


BOOL Dlg_OnInitDialog (HWND hwnd, HWND hwndFocus,
   LPARAM lParam) {

   TCHAR szBuf[50];
   SYSTEM_INFO si;

   // Associate an icon with the dialog box.
   SetClassLong(hwnd, GCL_HICON, (LONG)
      LoadIcon((HINSTANCE) GetWindowLong(hwnd, GWL_HINSTANCE),
         __TEXT("SysInfo")));

   GetSystemInfo(&si);

   // Fill the static controls in the
   // list box with the appropriate number.
   SetDlgItemText(hwnd, IDC_OEMID,
      BigNumToString(si.dwOemId, szBuf));

   SetDlgItemText(hwnd, IDC_PAGESIZE,
      BigNumToString(si.dwPageSize, szBuf));

   SetDlgItemText(hwnd, IDC_MINAPPADDR,
      BigNumToString((LONG) si.lpMinimumApplicationAddress,
      szBuf));

   SetDlgItemText(hwnd, IDC_MAXAPPADDR,
      BigNumToString((LONG) si.lpMaximumApplicationAddress,
      szBuf));

   _stprintf(szBuf, __TEXT("0x%08X"),
      si.dwActiveProcessorMask);
   SetDlgItemText(hwnd, IDC_ACTIVEPROCMASK, szBuf);

   SetDlgItemText(hwnd, IDC_NUMOFPROCS,
      BigNumToString(si.dwNumberOfProcessors, szBuf));

   SetDlgItemText(hwnd, IDC_PROCTYPE,
      GetFlagStr(si.dwProcessorType, CPUFlags, szBuf));

   SetDlgItemText(hwnd, IDC_ALLOCGRAN,
      BigNumToString(si.dwAllocationGranularity, szBuf));

   return(TRUE);
}


///////////////////////////////////////////////////////////


void Dlg_OnCommand (HWND hwnd, int id, HWND hwndCtl,
   UINT codeNotify) {

   switch (id) {
      case IDCANCEL:
         EndDialog(hwnd, id);
         break;
   }
}


///////////////////////////////////////////////////////////


BOOL CALLBACK Dlg_Proc (HWND hDlg, UINT uMsg,
   WPARAM wParam, LPARAM lParam) {

   BOOL fProcessed = TRUE;

   switch (uMsg) {
      HANDLE_MSG(hDlg, WM_INITDIALOG, Dlg_OnInitDialog);
      HANDLE_MSG(hDlg, WM_COMMAND, Dlg_OnCommand);

      default:
         fProcessed = FALSE;
         break;
   }
   return(fProcessed);
}


///////////////////////////////////////////////////////////


int WINAPI WinMain (HINSTANCE hinstExe,
   HINSTANCE hinstPrev, LPSTR lpszCmdLine, int nCmdShow) {

   DialogBox(hinstExe, MAKEINTRESOURCE(IDD_SYSINFO),
      NULL, Dlg_Proc);

   return(0);
}


/////////////////////// End Of File ///////////////////////
