/************************************************************
Module name: SomeLib.C
Notices: Copyright (c) 1995 Jeffrey Richter
************************************************************/


#include "..\AdvWin32.H"    /* See Appendix B for details. */
#include <windows.h>

#pragma warning(disable: 4001)      /* Single-line comment */

#include "SomeLib.RH"


#define _SOMELIBLIB_
#include "SomeLib.H"


/////////////////////////////////////////////////////////////


DWORD g_dwTlsIndex = TLS_OUT_OF_INDEXES;

// Per mapping instance data for this DLL
HINSTANCE g_hinstDll = NULL;


/////////////////////////////////////////////////////////////


BOOL WINAPI DllMain (HINSTANCE hinstDll, DWORD fdwReason,
   LPVOID lpvReserved) {

   LPTSTR lpszStr;

   switch (fdwReason) {

      case DLL_PROCESS_ATTACH:
         // DLL is attaching to the address space
         // of the current process.
         g_hinstDll = hinstDll;

         // Allocate a thread-local storage index.
         g_dwTlsIndex = TlsAlloc();

         if (g_dwTlsIndex == TLS_OUT_OF_INDEXES) {
            // The TLS index couldn't be allocated--have the
            // DLL return that initialization was
            // NOT successful.
            return(FALSE);
         }

         break;

      case DLL_THREAD_ATTACH:
         // A new thread is being created in the process.
         break;

      case DLL_THREAD_DETACH:
         // A thread is exiting cleanly.

         // Ensure that the TLS index was
         // allocated successfully.
         if (g_dwTlsIndex != TLS_OUT_OF_INDEXES) {

            // Get the pointer to the allocated memory.
            lpszStr = TlsGetValue(g_dwTlsIndex);

            // Test whether memory was ever allocated
            // for this thread.
            if (lpszStr != NULL) {
               HeapFree(GetProcessHeap(), 0, lpszStr);
            }
         }
         break;

      case DLL_PROCESS_DETACH:
         // The calling process is detaching the DLL
         // from its address space.

         // Ensure that the TLS index was
         // allocated successfully.
         if (g_dwTlsIndex != TLS_OUT_OF_INDEXES) {

            // Get the pointer to the allocated memory.
            lpszStr = TlsGetValue(g_dwTlsIndex);

            // Test whether memory was ever allocated
            // for this thread.
            if (lpszStr != NULL) {
               HeapFree(GetProcessHeap(), 0, lpszStr);
            }

            // Free the TLS index.
            TlsFree(g_dwTlsIndex);
         }
         break;
   }
   return(TRUE);
}


/////////////////////////////////////////////////////////////


#define RESSTR_SIZE     (1000 * sizeof(TCHAR))

SOMELIBAPI LPCTSTR LoadResString (void) {
   static int nStringId = 0;

   LPTSTR lpszStr = TlsGetValue(g_dwTlsIndex);

   if (lpszStr == NULL) {
      lpszStr = HeapAlloc(GetProcessHeap(), 0, RESSTR_SIZE);
      TlsSetValue(g_dwTlsIndex, lpszStr);
   }

   LoadString(g_hinstDll, IDS_STRINGFIRST + nStringId,
      lpszStr, RESSTR_SIZE);

   nStringId = (nStringId + 1) % IDS_STRINGNUM;

   return(lpszStr);
}


//////////////////////// End Of File ////////////////////////
