/************************************************************
Module name: VMStat.C
Notices: Copyright (c) 1995 Jeffrey Richter
************************************************************/


#include "..\AdvWin32.H"    /* See Appendix B for details. */
#include <windows.h>
#include <windowsx.h>

#pragma warning(disable: 4001)      /* Single-line comment */

#include <tchar.h>
#include "Resource.H"


/////////////////////////////////////////////////////////////


// This function accepts a number and converts it to a string,
// inserting commas where appropriate.
LPTSTR WINAPI BigNumToString (LONG lNum, LPTSTR szBuf) {
   WORD wNumDigits = 0, wNumChars = 0;

   do {
      // Put the last digit of the string
      // in the character buffer.
      szBuf[wNumChars++] = (TCHAR) (lNum % 10 + __TEXT('0'));

      // Increment the number of digits
      // that we put in the string.
      wNumDigits++;

      // For every three digits put in
      // the string, add a comma (,).
      if (wNumDigits % 3 == 0)
         szBuf[wNumChars++] = __TEXT(',');

      // Divide the number by 10, and repeat the process.
      lNum /= 10;
      // Continue adding digits to
      // the string until the number is zero.
   } while (lNum != 0);

   // If the last character added to
   // the string was a comma, truncate it.
   if (szBuf[wNumChars - 1] == __TEXT(','))
      szBuf[wNumChars - 1] = 0;

   // Ensure that the string is zero-terminated.
   szBuf[wNumChars] = 0;

   // We added all the characters to the string in reverse 
   // order. We must reverse the contents of the string.
   _tcsrev(szBuf);

   // Returns the address of the string.  This is the same
   // value that was passed to us initially.  Returning it
   // here makes it easier for the calling function
   // to use the string.
   return(szBuf);
}


/////////////////////////////////////////////////////////////


BOOL Dlg_OnInitDialog (HWND hwnd, HWND hwndFocus,
   LPARAM lParam) {

   TCHAR szBuf[50];
   MEMORYSTATUS ms;

    // Associate an icon with the dialog box.
    SetClassLong(hwnd, GCL_HICON, (LONG)
        LoadIcon((HINSTANCE) GetWindowLong(hwnd, GWL_HINSTANCE),
            (LPCTSTR) __TEXT("VMStat")));

   // Initialize the structure length before
   // calling GlobalMemoryStatus.
   ms.dwLength = sizeof(ms);
   GlobalMemoryStatus(&ms);

   // Fill the static controls in the
   // list box with the appropriate number.
   SetDlgItemText(hwnd, IDC_MEMLOAD,
      BigNumToString(ms.dwMemoryLoad, szBuf));

   SetDlgItemText(hwnd, IDC_TOTALPHYS,
      BigNumToString(ms.dwTotalPhys, szBuf));

   SetDlgItemText(hwnd, IDC_AVAILPHYS,
      BigNumToString(ms.dwAvailPhys, szBuf));

   SetDlgItemText(hwnd, IDC_TOTALPAGEFILE,
      BigNumToString(ms.dwTotalPageFile, szBuf));

   SetDlgItemText(hwnd, IDC_AVAILPAGEFILE,
      BigNumToString(ms.dwAvailPageFile, szBuf));

   SetDlgItemText(hwnd, IDC_TOTALVIRTUAL,
      BigNumToString(ms.dwTotalVirtual, szBuf));

   SetDlgItemText(hwnd, IDC_AVAILVIRTUAL,
      BigNumToString(ms.dwAvailVirtual, szBuf));

   return(TRUE);
}


/////////////////////////////////////////////////////////////


void Dlg_OnCommand (HWND hwnd, int id, HWND hwndCtl,
   UINT codeNotify) {

   switch (id) {
      case IDCANCEL:
         EndDialog(hwnd, id);
         break;
   }
}


/////////////////////////////////////////////////////////////


BOOL CALLBACK Dlg_Proc (HWND hDlg, UINT uMsg,
   WPARAM wParam, LPARAM lParam) {

   BOOL fProcessed = TRUE;

   switch (uMsg) {
      HANDLE_MSG(hDlg, WM_INITDIALOG, Dlg_OnInitDialog);
      HANDLE_MSG(hDlg, WM_COMMAND, Dlg_OnCommand);

      default:
         fProcessed = FALSE;
         break;
   }
   return(fProcessed);
}


/////////////////////////////////////////////////////////////


int WINAPI WinMain (HINSTANCE hinstExe,
   HINSTANCE hinstPrev, LPSTR lpszCmdLine, int nCmdShow) {

   DialogBox(hinstExe, MAKEINTRESOURCE(IDD_VMSTAT),
      NULL, Dlg_Proc);

   return(0);
}


//////////////////////// End Of File ////////////////////////
